package com.javacodegeeks.reactivestreams

import akka.NotUsed
import akka.actor.ActorSystem
import akka.stream.ActorMaterializer
import akka.stream.BidiShape
import akka.stream.scaladsl.BidiFlow
import akka.stream.scaladsl.GraphDSL
import akka.util.ByteString
import akka.stream.scaladsl.Flow
import akka.stream.scaladsl.Sink
import akka.stream.scaladsl.Source
import scala.concurrent.Future

object BidiFlows extends App {
  implicit val system = ActorSystem("reactive-streams")
  implicit val materializer = ActorMaterializer()
  implicit val executionContext = system.dispatcher
  
  case class Request(payload: ByteString)
  case class Response(payload: ByteString)
  
  val server = GraphDSL.create() { implicit builder => 
    import GraphDSL.Implicits._
   
    val out = builder.add(Flow[Request].map { _.payload.utf8String })
    val in = builder.add(Flow[String].map { s => Response(ByteString(s.reverse)) })

    BidiShape.fromFlows(out, in)
  }
  
  val bidiFlow = BidiFlow.fromGraph(server)
  
  Source
    .single(Request(ByteString("BidiFlow Example")))
    .via(bidiFlow.join(Flow[String]))
    .map(_.payload.utf8String)
    .runWith(Sink.foreach { println _ })
    .onComplete { _ => system.terminate() }
}
