package com.javacodegeeks.reactivestreams

import java.nio.file.Path
import java.nio.file.Paths

import akka.actor.ActorSystem
import akka.stream.ActorMaterializer
import akka.stream.scaladsl.FileIO
import akka.stream.scaladsl.Framing
import akka.util.ByteString
import akka.stream.scaladsl.Flow
import akka.stream.scaladsl.Source
import akka.stream.scaladsl.Sink
import akka.stream.scaladsl.Keep
import scala.concurrent.Future
import scala.util.Success
import scala.util.Failure

object Flows extends App {
  implicit val system = ActorSystem("reactive-streams")
  implicit val materializer = ActorMaterializer()
  implicit val executionContext = system.dispatcher
  
  val file: Path = Paths.get(getClass.getResource("/numbers.txt").toURI())
  val source: Source[ByteString, _] = FileIO.fromPath(file) 
  
  val flow: Flow[ByteString, Int, _] = Flow[ByteString]
    .via(Framing.delimiter(ByteString("\r\n"), 10, true))
    .map { _.utf8String.toInt } 
      
  source
    .via(flow)
    .filter { _ > 200 }
    .runForeach { println _ }
 
  val future: Future[Int] = source
    .via(flow)
    .toMat(Sink.fold[Int, Int](0){(acc, _) => acc + 1 })(Keep.right)
    .run    
  
  future.onSuccess { case count => 
    println(s"Lines processed: $count") 
  }
  
  future.onComplete { _ => system.terminate() }
}
