package com.javacodegeeks.reactivestreams

import java.nio.file.Path
import java.nio.file.Paths

import akka.actor.ActorSystem
import akka.stream.ActorMaterializer
import akka.stream.scaladsl.FileIO
import akka.stream.scaladsl.Framing
import akka.util.ByteString
import akka.stream.scaladsl.Flow
import akka.stream.scaladsl.Source
import akka.stream.scaladsl.Sink
import akka.stream.scaladsl.Keep
import scala.concurrent.Future
import scala.util.Success
import scala.util.Failure
import akka.stream.scaladsl.RunnableGraph
import akka.stream.scaladsl.GraphDSL
import akka.NotUsed
import akka.stream.scaladsl.Broadcast
import akka.stream.scaladsl.Merge
import akka.stream.ClosedShape

object Graphs extends App {
  implicit val system = ActorSystem("reactive-streams")
  implicit val materializer = ActorMaterializer()
  implicit val executionContext = system.dispatcher
  
  case class Employee(name: String, position: String, salary: Int)
  
  val employees = List(
    Employee("Tom", "manager", 50000),
    Employee("Bob", "employee", 20000),
    Employee("Mark", "employee", 20000),
    Employee("John", "manager", 55000),
    Employee("Dilan", "employee", 35000)      
  )

  val graph = GraphDSL.create() { implicit builder => 
    import GraphDSL.Implicits._
   
    val in = Source(employees)
    val out = Sink.foreach { println _ }

    val broadcast = builder.add(Broadcast[Employee](2))
    val merge = builder.add(Merge[Employee](2))

    val manager = Flow[Employee]
      .filter { _.position == "manager" }
      .map { e => e.copy(salary = e.salary + 10000) } 
    
    val employee = Flow[Employee]
      .filter { _.position != "manager" }
      .map { e => e.copy(salary = e.salary + 5000) }

    in ~> broadcast ~> manager  ~> merge ~> out
          broadcast ~> employee ~> merge
          
    ClosedShape
  }
  
  RunnableGraph
    .fromGraph(graph)
    .run
}
