package com.javacodegeeks.reactivestreams

import scala.concurrent.ExecutionContext.Implicits.global

import org.specs2.matcher.FutureMatchers
import org.specs2.mutable.Specification
import org.specs2.mutable.SpecificationLike

import akka.actor.ActorSystem
import akka.stream.ActorMaterializer
import akka.stream.scaladsl.Flow
import akka.testkit.TestKit
import akka.util.ByteString
import akka.stream.scaladsl.Framing
import akka.stream.testkit.scaladsl.TestSource
import akka.stream.testkit.scaladsl.TestSink
import akka.stream.scaladsl.Keep
import org.specs2.specification.AfterAll

class FlowsSpec extends TestKit(ActorSystem("reactive-streams-test")) 
    with SpecificationLike with FutureMatchers with AfterAll {
  
  implicit val materializer = ActorMaterializer()

  val flow: Flow[ByteString, Int, _] = Flow[ByteString]
    .via(Framing.delimiter(ByteString("\r\n"), 10, true))
    .map { _.utf8String.toInt } 
  
  def afterAll = system.terminate()

  "Stream" >> {
    "should return an expected value" >> {   
      val (publisher, subscriber) = TestSource.probe[ByteString]
        .via(flow)
        .toMat(TestSink.probe[Int])(Keep.both)
        .run()
        
      subscriber.request(2)
      publisher.sendNext(ByteString("20"))  
      publisher.sendNext(ByteString("0\r\n"))
      publisher.sendComplete()
      
      subscriber.expectNext() must be_==(200)
    }
  }
}